#include "config.h"
#include <stdlib.h>

int i=20;
static unsigned long int next = 1;
int RRrand()
   {
   next = next * 1103515245L + 12345L;
   return ((int) (next / 0x10000L) & 0x7FFF);
   }

void RRsrand(seed)
   unsigned int seed;
   {
   next = seed;
   }
main()
{
atexit(0);
seednrand(1);
while(i--)
{
printf("%lf \n", nrand());
}
}

