/* rle_config.h
 * 
 * Automatically generated by make-config-h script.
 * DO NOT EDIT THIS FILE.
 * Edit include/makefile.src and the configuration file instead.
 */

#define SUNTOOLS SUNTOOLS
#define ABEKASA60 ABEKASA60
#define ABEKASA62 ABEKASA62
#define CUBICOMP CUBICOMP
#define GIF GIF
#define GRAYFILES GRAYFILES
#define MACPAINT MACPAINT
#define POSTSCRIPT POSTSCRIPT
#define SUNRASTER SUNRASTER
#define TARGA TARGA
#define TIFF TIFF
#define WASATCH WASATCH
#define WAVEFRONT WAVEFRONT
#define NO_MAKE_MAKEFILE NO_MAKE_MAKEFILE
#define SYS_TIME_H SYS_TIME_H
#define USE_RANDOM USE_RANDOM
/***************** From rle_config.tlr *****************/

/* CONST_DECL must be defined as 'const' or nothing. */
#ifdef CONST_DECL
#undef CONST_DECL
#define CONST_DECL const

#else
#define CONST_DECL

#endif

/* A define for getx11. */
#ifndef XLIBINT_H
#define XLIBINT_H_NOT_AVAILABLE
#endif
