/*
 * picture.h
 * Revision 4.0  91/07/17  14:47:07  kolb
 * Initial version.
 */
#ifndef PICTURE_H
#define PICTURE_H
#define CORRECT(x)		(correct(GAMMACORRECT(x)))
#define GAMMACORRECT(x)		(Options.gamma == 1. ? (x) \
				   : (pow(x, Options.gamma)))
/*
 * Pixel
 */
typedef struct Pixel {
	Float r, g, b, alpha;		/* Red, green, blue, alpha channel */
} Pixel;
 
extern void	PictureStart(), PictureEnd(), PictureWriteLine();
 
#ifdef URT
extern void	PictureSetWindow();
#include "u_.h"
#include "u_raw.h"
#endif
 
#endif /* PICTURE_H */
