/*
 * $Log:	funcdefs.h,v $
 * Revision 4.0  91/07/17  14:46:11  kolb
 * Initial version.
 * 
 */
#ifndef FUNCDEFS_H
#define FUNCDEFS_H
 
#ifdef I_STRING
#include <string.h>
#else
#include <strings.h>
#endif
 
/*
 * Misc. routines.
 */
extern void	get_cpu_time(),
		read_input_file();
 
#endif /* FUNCDEFS_H */
