/*	@(#)svc_auth.h 1.9 89/07/26 SMI	*/

/*
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

/*
 * Service side of rpc authentication.
 */

#ifndef _rpc_svc_auth_h
#define	_rpc_svc_auth_h

extern enum auth_stat _authenticate();

#endif /*!_rpc_svc_auth_h*/
