#include "rrloops.h"
#include <stdio.h>
#include <time.h>
#ifdef _BSD
struct timeval sucktime={5,0};
void GetMyHostname(s)
	char **s;
{
	char s2[MAXHOSTNAMELEN];
	if( gethostname(s2,MAXHOSTNAMELEN)) {
		fprintf(stderr,"can`t get my hostname\n");
		exit(1);
	}
	*s=(char*)strsave(s2);
}
void RRSuckChannels()
{
	static char suckbuf[1024];
	fd_set readfds;
	int dtblsize=_rpc_dtablesize();
	
	while(TRUE) {
		readfds=svc_fdset;
		switch(select(dtblsize,&readfds, (fd_set*) 0, (fd_set*) 0, &sucktime)){

			case -1:
				return;
			case 0:
				return;	
			default: 
			
				;
		}
	}
}

#endif
	


