/*
 * mist.h
 * Revision 4.0  91/07/17  14:40:46  kolb
 * Initial version.
 */
#ifndef MIST_H
#define MIST_H 
#define AtmosMistCreate(c,t,z,s) AtmosCreate((AtmosRef)MistCreate(c,t,z,s), \
						MistApply)
typedef struct Mist {
	Color color;		/* Mist color */
	Color trans;		/* R, G, B transmissivity */
	Float scale, zero;	/* Height scale, start Z */
} Mist;
 
extern Mist *MistCreate();
extern void MistApply();
 
#endif /* MIST_H */
 
