/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <rpc/types.h>

#ifndef RRPROG_H
#define RRPROG_H
#include "config.h"
#include "progspec.h"
#include <librr/rrpc.h>

typedef double rpc_RTable[267];
bool_t xdr_rpc_RTable();

struct rpc_ExprAssoc {
	u_long lhs;
	rpc_Expr *expr;
	struct rpc_ExprAssoc *next;
};
typedef struct rpc_ExprAssoc rpc_ExprAssoc;
bool_t xdr_rpc_ExprAssoc();

struct rpc_Vec2d {
	double u;
	double v;
};
typedef struct rpc_Vec2d rpc_Vec2d;
bool_t xdr_rpc_Vec2d();

struct rpc_Vector {
	double x;
	double y;
	double z;
};
typedef struct rpc_Vector rpc_Vector;
bool_t xdr_rpc_Vector();

struct rpc_RSMatrix {
	double matrix[9];
	rpc_Vector translate;
};
typedef struct rpc_RSMatrix rpc_RSMatrix;
bool_t xdr_rpc_RSMatrix();

struct rpc_Color {
	double r;
	double g;
	double b;
};
typedef struct rpc_Color rpc_Color;
bool_t xdr_rpc_Color();

typedef rpc_Color rpc_ColorMap[256];
bool_t xdr_rpc_ColorMap();

struct rpc_GeomList {
	struct rpc_Geom *obj;
	struct rpc_GeomList *next;
};
typedef struct rpc_GeomList rpc_GeomList;
bool_t xdr_rpc_GeomList();

struct rpc_Instance {
	rpc_Geom *obj;
	double bounds[6];
};
typedef struct rpc_Instance rpc_Instance;
bool_t xdr_rpc_Instance();

struct rpc_uInstance {
	long obj;
	double bounds[6];
};
typedef struct rpc_uInstance rpc_uInstance;
bool_t xdr_rpc_uInstance();

struct rpc_List {
	rpc_Geom *list;
	rpc_Geom *unbounded;
	double bounds[6];
};
typedef struct rpc_List rpc_List;
bool_t xdr_rpc_List();

struct rpc_Rotate {
	double x;
	double y;
	double z;
	double theta;
};
typedef struct rpc_Rotate rpc_Rotate;
bool_t xdr_rpc_Rotate();

struct rpc_MetaVector {
	double rs;
	double c4;
	double c2;
	double c0;
	double x;
	double y;
	double z;
};
typedef struct rpc_MetaVector rpc_MetaVector;
bool_t xdr_rpc_MetaVector();

struct rpc_Box {
	double bounds[6];
};
typedef struct rpc_Box rpc_Box;
bool_t xdr_rpc_Box();

struct rpc_Cone {
	double start_dist;
	rpc_Trans trans;
};
typedef struct rpc_Cone rpc_Cone;
bool_t xdr_rpc_Cone();

struct rpc_Cylinder {
	rpc_Trans trans;
};
typedef struct rpc_Cylinder rpc_Cylinder;
bool_t xdr_rpc_Cylinder();

struct rpc_Disc {
	double radius;
	double d;
	rpc_Vector norm;
	rpc_Vector pos;
};
typedef struct rpc_Disc rpc_Disc;
bool_t xdr_rpc_Disc();

struct rpc_Plane {
	rpc_Vector norm;
	rpc_Vector pos;
	double d;
};
typedef struct rpc_Plane rpc_Plane;
bool_t xdr_rpc_Plane();

struct rpc_Sphere {
	double r;
	double rsq;
	double x;
	double y;
	double z;
};
typedef struct rpc_Sphere rpc_Sphere;
bool_t xdr_rpc_Sphere();

struct rpc_Torus {
	double a;
	double b;
	double aa;
	double bb;
	rpc_Trans trans;
};
typedef struct rpc_Torus rpc_Torus;
bool_t xdr_rpc_Torus();

struct rpc_Triangle {
	rpc_Vector nrm;
	rpc_Vector p[3];
	rpc_Vector e[3];
	rpc_Vector *vnorm;
	rpc_Vector *dpdu;
	rpc_Vector *dpdv;
	double d;
	double b[3];
	rpc_Vec2d *uv;
	char index;
	char type;
};
typedef struct rpc_Triangle rpc_Triangle;
bool_t xdr_rpc_Triangle();

struct rpc_ShadowCache {
	struct rpc_Geom *obj;
	rpc_RSMatrix trans;
	char dotrans;
};
typedef struct rpc_ShadowCache rpc_ShadowCache;
bool_t xdr_rpc_ShadowCache();

struct rpc_Pointlight {
	rpc_Vector pos;
};
typedef struct rpc_Pointlight rpc_Pointlight;
bool_t xdr_rpc_Pointlight();

struct rpc_Spotlight {
	rpc_Vector pos;
	rpc_Vector dir;
	double coef;
	double radius;
	double falloff;
};
typedef struct rpc_Spotlight rpc_Spotlight;
bool_t xdr_rpc_Spotlight();

struct rpc_Extended {
	rpc_Vector pos;
	double radius;
};
typedef struct rpc_Extended rpc_Extended;
bool_t xdr_rpc_Extended();

struct rpc_Jittered {
	rpc_Vector pos;
	rpc_Vector curpos;
	rpc_Vector e1;
	rpc_Vector e2;
};
typedef struct rpc_Jittered rpc_Jittered;
bool_t xdr_rpc_Jittered();

struct rpc_Infinite {
	rpc_Vector dir;
};
typedef struct rpc_Infinite rpc_Infinite;
bool_t xdr_rpc_Infinite();

struct rpc_Surface {
	nm name;
	rpc_Color amb;
	rpc_Color diff;
	rpc_Color spec;
	rpc_Color translu;
	rpc_Color body;
	double srexp;
	double stexp;
	double statten;
	double index;
	double reflect;
	double transp;
	double translucency;
	char noshadow;
	struct rpc_Surface *next;
};
typedef struct rpc_Surface rpc_Surface;
bool_t xdr_rpc_Surface();

struct rpc_Fog {
	rpc_Color color;
	rpc_Color trans;
};
typedef struct rpc_Fog rpc_Fog;
bool_t xdr_rpc_Fog();

struct rpc_Fogdeck {
	double alt;
	double offset;
	double chaoscale;
	rpc_Vector scale;
	rpc_Color color;
	rpc_Color trans;
	int octaves;
};
typedef struct rpc_Fogdeck rpc_Fogdeck;
bool_t xdr_rpc_Fogdeck();

struct rpc_Mist {
	rpc_Color color;
	rpc_Color trans;
	double scale;
	double zero;
};
typedef struct rpc_Mist rpc_Mist;
bool_t xdr_rpc_Mist();

struct rpc_Blotch {
	double mix;
	rpc_uSurface_Ptr surf;
};
typedef struct rpc_Blotch rpc_Blotch;
bool_t xdr_rpc_Blotch();

struct rpc_Bump {
	double size;
};
typedef struct rpc_Bump rpc_Bump;
bool_t xdr_rpc_Bump();

struct rpc_Checker {
	rpc_Surface *surf;
};
typedef struct rpc_Checker rpc_Checker;
bool_t xdr_rpc_Checker();

struct rpc_CloudText {
	double beta;
	double omega;
	double lambda;
	double scale;
	double cthresh;
	double range;
	double transcale;
	double maxval;
	int octaves;
};
typedef struct rpc_CloudText rpc_CloudText;
bool_t xdr_rpc_CloudText();

struct rpc_FBm {
	double beta;
	double omega;
	double lambda;
	double scale;
	double offset;
	double thresh;
	int octaves;
	rpc_ColorMap *colormap;
};
typedef struct rpc_FBm rpc_FBm;
bool_t xdr_rpc_FBm();

struct rpc_Gloss {
	double glossy;
};
typedef struct rpc_Gloss rpc_Gloss;
bool_t xdr_rpc_Gloss();

struct rpc_ImageText {
	u_long image;
	rpc_Surface *surf;
	int component;
	int smooth;
	double lo;
	double hi;
	double tileu;
	double tilev;
	rpc_Mapping *mapping;
};
typedef struct rpc_ImageText rpc_ImageText;
bool_t xdr_rpc_ImageText();

struct rpc_Marble {
	rpc_ColorMap *colormap;
};
typedef struct rpc_Marble rpc_Marble;
bool_t xdr_rpc_Marble();

struct rpc_Mount {
	double turb;
	double slope;
	rpc_ColorMap *cmap;
};
typedef struct rpc_Mount rpc_Mount;
bool_t xdr_rpc_Mount();

struct rpc_Sky {
	double beta;
	double omega;
	double lambda;
	double scale;
	double cthresh;
	double lthresh;
	int octaves;
};
typedef struct rpc_Sky rpc_Sky;
bool_t xdr_rpc_Sky();

struct rpc_Stripe {
	rpc_uSurface_Ptr surf;
	rpc_Mapping *mapping;
	double width;
	double bump;
};
typedef struct rpc_Stripe rpc_Stripe;
bool_t xdr_rpc_Stripe();

struct rpc_WindyText {
	double scale;
	double windscale;
	double chaoscale;
	double bumpscale;
	double tscale;
	double hscale;
	double offset;
	int octaves;
};
typedef struct rpc_WindyText rpc_WindyText;
bool_t xdr_rpc_WindyText();

struct rpc_Medium {
	double index;
	double statten;
	struct rpc_Medium *next;
};
typedef struct rpc_Medium rpc_Medium;
bool_t xdr_rpc_Medium();

struct rpc_RSOptions {
	int stereo;
	int verbose;
	int quiet;
	int jitter;
	int samples;
	int maxdepth;
	int report_freq;
	int no_shadows;
	int shadowtransp;
	int cache;
	int appending;
	int resolution_set;
	int contrast_set;
	int samples_set;
	int cutoff_set;
	int maxdepth_set;
	int window_set;
	int freq_set;
	int jitter_set;
	int eyesep_set;
	int csg;
	int flipnorm;
	int samplemap;
	int gaussian;
	int framenum;
	int startframe;
	int endframe;
	int totalframes;
	int totalframes_set;
	int alpha;
	int exp_output;
	double eyesep;
	double gamma;
	double xmin;
	double xmax;
	double ymin;
	double ymax;
	double starttime;
	double shutterspeed;
	double framestart;
	double framelength;
	double filterwidth;
	rpc_Color contrast;
	rpc_Color cutoff;
	rpc_Color ambient;
	nm progname;
	nm statsname;
	nm imgname;
	nm inputname;
	nm cppargs;
	int is_rpc_client;
	int is_rpc_server;
	int squaresize;
	int display;
	nm serverfilename;
	nm serverexestring;
	long pictfile;
};
typedef struct rpc_RSOptions rpc_RSOptions;
bool_t xdr_rpc_RSOptions();

struct rpc_RSScreen {
	int xres;
	int yres;
	int xsize;
	int ysize;
	int minx;
	int miny;
	int maxx;
	int maxy;
	rpc_Vector scrnx;
	rpc_Vector scrny;
	rpc_Vector scrni;
	rpc_Vector scrnj;
	rpc_Vector firstray;
	rpc_Color background;
};
typedef struct rpc_RSScreen rpc_RSScreen;
bool_t xdr_rpc_RSScreen();

struct rpc_RSCamera {
	rpc_Vector pos;
	rpc_Vector lookp;
	rpc_Vector dir;
	rpc_Vector up;
	double hfov;
	double vfov;
	double lookdist;
	double aperture;
	double focaldist;
};
typedef struct rpc_RSCamera rpc_RSCamera;
bool_t xdr_rpc_RSCamera();
#endif
