/*	@(#)raw.h 1.2 89/07/26 SMI	*/

/*
 * raw.h
 *
 * Raw interface
 * The common memory area over which they will communicate
 *
 * Copyright (C) 1989, Sun Microsystems, Inc.
 */

#ifndef _RPC_RAW_H
#define	_RPC_RAW_H

char *_rpcrawcombuf;
#endif /* !_RPC_RAW_H */
