/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */
#ifndef RPCSPEC_H
#define RPCSPEC_H


#ifdef ONCRPC
#include <rpc/rpc.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netdb.h>
#include <rpc/pmap_prot.h>
#include <rpc/pmap_clnt.h>
#include <sys/param.h>
#define MAXPROGNUMS 1024
#define ACTION_ON_CALLBACK  svc_getreqset(&readfds)
typedef u_long RRCallerID;
extern void remoterender_1();
extern void DispatchCallback();
extern void ConnectionInfo();
#endif

#endif /* RPCSPEC_H */

