#undef UnitCirclePoint
 #define UnitCirclePoint  G0
 #undef yyprevious
 #define yyprevious  G1
 #undef yyfilename
 #define yyfilename  G2
 #undef yyfilename
 #define yyfilename  G3
 #undef yyfilename
 #define yyfilename  G4
 #undef  Defstack
 #define  Defstack  G5
 #undef OpenStatsFile
 #define OpenStatsFile  G6
 #undef  Defstack
 #define  Defstack  G7
 #undef TopMedium
 #define TopMedium  G8
 #undef NoiseInit
 #define NoiseInit  G9
 #undef OpenStatsFile
 #define OpenStatsFile  G10
 #undef ShadowOptions
 #define ShadowOptions  G11
 #undef SpotAtten
 #define SpotAtten  G12
 #undef PrintMemoryStats
 #define PrintMemoryStats  G13
 #undef GeomStats
 #define GeomStats  G14
 #undef UnitCirclePoint
 #define UnitCirclePoint  G15
 #undef focus_blur_ray
 #define focus_blur_ray  G16
 #undef  AtmosEffects
 #define  AtmosEffects  G17
 #undef TopMedium
 #define TopMedium  G18
 #undef GeomAddToDefined
 #define GeomAddToDefined  G19
 #undef LightAddToDefined
 #define LightAddToDefined  G20
 #undef SurfaceAddToDefined
 #define SurfaceAddToDefined  G21
 #undef  SurfaceGetNamed
 #define  SurfaceGetNamed  G22
 #undef  GeomGetNamed
 #define  GeomGetNamed  G23
 #undef yyerrflag
 #define yyerrflag  G24
 #undef AreaLightCreate
 #define AreaLightCreate  G25
 #undef AreaLightCreate
 #define AreaLightCreate  G26
 #undef DefaultSurface
 #define DefaultSurface  G27
 #undef DefaultSurface
 #define DefaultSurface  G28
 #undef  MediumPush
 #define  MediumPush  G29
 #undef  AtmosCreate
 #define  AtmosCreate  G30
 #undef  AtmosphereCopy
 #define  AtmosphereCopy  G31
 #undef Atmospherics
 #define Atmospherics  G32
 #undef BlobIntersect
 #define BlobIntersect  G33
 #undef BlobNormal
 #define BlobNormal  G34
 #undef BlobBounds
 #define BlobBounds  G35
 #undef BlobStats
 #define BlobStats  G36
 #undef  BlobCreate
 #define  BlobCreate  G37
 #undef  BlobName
 #define  BlobName  G38
 #undef  BlobMethods
 #define  BlobMethods  G39
 #undef  BlotchCreate
 #define  BlotchCreate  G40
 #undef BlotchApply
 #define BlotchApply  G41
 #undef BoundsCopy
 #define BoundsCopy  G42
 #undef BoundsPrint
 #define BoundsPrint  G43
 #undef BoundsInit
 #define BoundsInit  G44
 #undef BoundsEnlarge
 #define BoundsEnlarge  G45
 #undef BoundsTransform
 #define BoundsTransform  G46
 #undef BoundsIntersect
 #define BoundsIntersect  G47
 #undef  BoxCreate
 #define  BoxCreate  G48
 #undef BoxIntersect
 #define BoxIntersect  G49
 #undef BoxNormal
 #define BoxNormal  G50
 #undef BoxBounds
 #define BoxBounds  G51
 #undef  BoxMethods
 #define  BoxMethods  G52
 #undef DivideExpr
 #define DivideExpr  G53
 #undef NegateExpr
 #define NegateExpr  G54
 #undef LinearTime
 #define LinearTime  G55
 #undef  BumpCreate
 #define  BumpCreate  G56
 #undef BumpApply
 #define BumpApply  G57
 #undef  CheckerCreate
 #define  CheckerCreate  G58
 #undef CheckerApply
 #define CheckerApply  G59
 #undef  CloudTextCreate
 #define  CloudTextCreate  G60
 #undef CloudTextApply
 #define CloudTextApply  G61
 #undef  share_malloc
 #define  share_malloc  G62
 #undef  share_calloc
 #define  share_calloc  G63
 #undef  ConeCreate
 #define  ConeCreate  G64
 #undef ConeIntersect
 #define ConeIntersect  G65
 #undef ConeEnter
 #define ConeEnter  G66
 #undef ConeNormal
 #define ConeNormal  G67
 #undef ConeBounds
 #define ConeBounds  G68
 #undef ConeStats
 #define ConeStats  G69
 #undef  ConeName
 #define  ConeName  G70
 #undef  ConeMethods
 #define  ConeMethods  G71
 #undef  CsgCreate
 #define  CsgCreate  G72
 #undef CsgIntersect
 #define CsgIntersect  G73
 #undef CsgConvert
 #define CsgConvert  G74
 #undef CsgBounds
 #define CsgBounds  G75
 #undef FirstCsgGeom
 #define FirstCsgGeom  G76
 #undef  CsgMethods
 #define  CsgMethods  G77
 #undef  CylinderCreate
 #define  CylinderCreate  G78
 #undef CylinderIntersect
 #define CylinderIntersect  G79
 #undef CylinderEnter
 #define CylinderEnter  G80
 #undef CylinderNormal
 #define CylinderNormal  G81
 #undef CylinderBounds
 #define CylinderBounds  G82
 #undef CylinderUV
 #define CylinderUV  G83
 #undef CylinderStats
 #define CylinderStats  G84
 #undef  CylinderName
 #define  CylinderName  G85
 #undef  CylinderMethods
 #define  CylinderMethods  G86
 #undef  DiscCreate
 #define  DiscCreate  G87
 #undef  DiscMethods
 #define  DiscMethods  G88
 #undef DiscIntersect
 #define DiscIntersect  G89
 #undef DiscEnter
 #define DiscEnter  G90
 #undef DiscNormal
 #define DiscNormal  G91
 #undef DiscBounds
 #define DiscBounds  G92
 #undef DiscStats
 #define DiscStats  G93
 #undef  DiscName
 #define  DiscName  G94
 #undef  ExprResolve1
 #define  ExprResolve1  G95
 #undef  ExprResolve2
 #define  ExprResolve2  G96
 #undef  ExprResolve3
 #define  ExprResolve3  G97
 #undef  ExprResolve4
 #define  ExprResolve4  G98
 #undef  ExprResolve5
 #define  ExprResolve5  G99
 #undef  ExprFloatCreate
 #define  ExprFloatCreate  G100
 #undef  TimeExpr
 #define  TimeExpr  G101
 #undef  FrameExpr
 #define  FrameExpr  G102
 #undef  ExprReuseFloatCreate
 #define  ExprReuseFloatCreate  G103
 #undef  AssocCreate
 #define  AssocCreate  G104
 #undef  ExtendedCreate
 #define  ExtendedCreate  G105
 #undef  ExtendedMethods
 #define  ExtendedMethods  G106
 #undef ExtendedIntens
 #define ExtendedIntens  G107
 #undef ExtendedDirection
 #define ExtendedDirection  G108
 #undef  FBmCreate
 #define  FBmCreate  G109
 #undef  FBmBumpCreate
 #define  FBmBumpCreate  G110
 #undef FBmBumpApply
 #define FBmBumpApply  G111
 #undef  FogCreate
 #define  FogCreate  G112
 #undef  FogdeckCreate
 #define  FogdeckCreate  G113
 #undef FogdeckApply
 #define FogdeckApply  G114
 #undef get_cpu_time
 #define get_cpu_time  G115
 #undef read_input_file
 #define read_input_file  G116
 #undef  GeomName
 #define  GeomName  G117
 #undef  GeomCreate
 #define  GeomCreate  G118
 #undef  GeomCopy
 #define  GeomCopy  G119
 #undef  GeomCopyNamed
 #define  GeomCopyNamed  G120
 #undef  GeomComputeAggregateBounds
 #define  GeomComputeAggregateBounds  G121
 #undef  GeomStackPush
 #define  GeomStackPush  G122
 #undef  GeomStackPop
 #define  GeomStackPop  G123
 #undef AggregatePrintInfo
 #define AggregatePrintInfo  G124
 #undef IntersectStats
 #define IntersectStats  G125
 #undef AggregateConvert
 #define AggregateConvert  G126
 #undef PrimNormal
 #define PrimNormal  G127
 #undef  MethodsCreate
 #define  MethodsCreate  G128
 #undef  GlossCreate
 #define  GlossCreate  G129
 #undef GlossApply
 #define GlossApply  G130
 #undef  GridName
 #define  GridName  G131
 #undef  GirdBounds
 #define  GirdBounds  G132
 #undef GridIntersect
 #define GridIntersect  G133
 #undef GridConvert
 #define GridConvert  G134
 #undef  GridCreate
 #define  GridCreate  G135
 #undef  GridMethods
 #define  GridMethods  G136
 #undef  HfCreate
 #define  HfCreate  G137
 #undef HfIntersect
 #define HfIntersect  G138
 #undef  HfMethods
 #define  HfMethods  G139
 #undef  ImageTextCreate
 #define  ImageTextCreate  G140
 #undef ImageTextApply
 #define ImageTextApply  G141
 #undef ImageTextSetComponent
 #define ImageTextSetComponent  G142
 #undef  InfiniteCreate
 #define  InfiniteCreate  G143
 #undef  InfiniteMethods
 #define  InfiniteMethods  G144
 #undef InfiniteIntens
 #define InfiniteIntens  G145
 #undef InfiniteDirection
 #define InfiniteDirection  G146
 #undef  InstanceName
 #define  InstanceName  G147
 #undef InstanceIntersect
 #define InstanceIntersect  G148
 #undef InstanceConvert
 #define InstanceConvert  G149
 #undef InstanceBounds
 #define InstanceBounds  G150
 #undef  InstanceCreate
 #define  InstanceCreate  G151
 #undef  InstanceMethods
 #define  InstanceMethods  G152
 #undef  JitteredCreate
 #define  JitteredCreate  G153
 #undef  JitteredMethods
 #define  JitteredMethods  G154
 #undef JitteredIntens
 #define JitteredIntens  G155
 #undef JitteredDirection
 #define JitteredDirection  G156
 #undef  LightMethodsCreate
 #define  LightMethodsCreate  G157
 #undef  LightCreate
 #define  LightCreate  G158
 #undef LightAllocateCache
 #define LightAllocateCache  G159
 #undef LightAddToDefined
 #define LightAddToDefined  G160
 #undef LightIntens
 #define LightIntens  G161
 #undef LightDirection
 #define LightDirection  G162
 #undef ShadowSetOptions
 #define ShadowSetOptions  G163
 #undef ShadowStats
 #define ShadowStats  G164
 #undef  ListName
 #define  ListName  G165
 #undef ListIntersect
 #define ListIntersect  G166
 #undef ListConvert
 #define ListConvert  G167
 #undef ListBounds
 #define ListBounds  G168
 #undef  ListCreate
 #define  ListCreate  G169
 #undef  ListMethods
 #define  ListMethods  G170
 #undef  UVMappingCreate
 #define  UVMappingCreate  G171
 #undef  SphereMappingCreate
 #define  SphereMappingCreate  G172
 #undef  CylMappingCreate
 #define  CylMappingCreate  G173
 #undef  LinearMappingCreate
 #define  LinearMappingCreate  G174
 #undef  MarbleCreate
 #define  MarbleCreate  G175
 #undef MarbleApply
 #define MarbleApply  G176
 #undef  MistCreate
 #define  MistCreate  G177
 #undef MistApply
 #define MistApply  G178
 #undef  MountCreate
 #define  MountCreate  G179
 #undef MountApply
 #define MountApply  G180
 #undef OptionsList
 #define OptionsList  G181
 #undef OptionsSet
 #define OptionsSet  G182
 #undef PictureStart
 #define PictureStart  G183
 #undef PictureEnd
 #define PictureEnd  G184
 #undef PictureWriteLine
 #define PictureWriteLine  G185
 #undef PictureSetWindow
 #define PictureSetWindow  G186
 #undef  PlaneCreate
 #define  PlaneCreate  G187
 #undef PlaneIntersect
 #define PlaneIntersect  G188
 #undef PlaneNormal
 #define PlaneNormal  G189
 #undef PlaneBounds
 #define PlaneBounds  G190
 #undef PlaneStats
 #define PlaneStats  G191
 #undef  PlaneName
 #define  PlaneName  G192
 #undef  PlaneMethods
 #define  PlaneMethods  G193
 #undef  PointCreate
 #define  PointCreate  G194
 #undef  PointMethods
 #define  PointMethods  G195
 #undef PointIntens
 #define PointIntens  G196
 #undef PointDirection
 #define PointDirection  G197
 #undef  PolygonCreate
 #define  PolygonCreate  G198
 #undef  PolygonMethods
 #define  PolygonMethods  G199
 #undef PolygonIntersect
 #define PolygonIntersect  G200
 #undef PolygonEnter
 #define PolygonEnter  G201
 #undef PolygonNormal
 #define PolygonNormal  G202
 #undef PolygonBounds
 #define PolygonBounds  G203
 #undef PolygonUV
 #define PolygonUV  G204
 #undef PolygonStats
 #define PolygonStats  G205
 #undef  PolygonName
 #define  PolygonName  G206
 #undef  RotateCreate
 #define  RotateCreate  G207
 #undef  RotateMethods
 #define  RotateMethods  G208
 #undef RotatePropagate
 #define RotatePropagate  G209
 #undef SamplingSetOptions
 #define SamplingSetOptions  G210
 #undef UnitCirclePoint
 #define UnitCirclePoint  G211
 #undef  ScaleCreate
 #define  ScaleCreate  G212
 #undef  ScaleMethods
 #define  ScaleMethods  G213
 #undef ScalePropagate
 #define ScalePropagate  G214
 #undef  SkyCreate
 #define  SkyCreate  G215
 #undef  SphereCreate
 #define  SphereCreate  G216
 #undef  SphereMethods
 #define  SphereMethods  G217
 #undef SphereIntersect
 #define SphereIntersect  G218
 #undef SphereEnter
 #define SphereEnter  G219
 #undef SphereNormal
 #define SphereNormal  G220
 #undef SphereBounds
 #define SphereBounds  G221
 #undef SphereStats
 #define SphereStats  G222
 #undef  SphereName
 #define  SphereName  G223
 #undef  SpotCreate
 #define  SpotCreate  G224
 #undef  SpotMethods
 #define  SpotMethods  G225
 #undef SpotIntens
 #define SpotIntens  G226
 #undef SpotDirection
 #define SpotDirection  G227
 #undef StatsPrint
 #define StatsPrint  G228
 #undef StatsAddRep
 #define StatsAddRep  G229
 #undef  StripeCreate
 #define  StripeCreate  G230
 #undef StripeApply
 #define StripeApply  G231
 #undef  SurfaceCreate
 #define  SurfaceCreate  G232
 #undef  GetShadingSurf
 #define  GetShadingSurf  G233
 #undef  SurfPush
 #define  SurfPush  G234
 #undef SurfaceBlend
 #define SurfaceBlend  G235
 #undef ColorBlend
 #define ColorBlend  G236
 #undef ComputeSurfProps
 #define ComputeSurfProps  G237
 #undef  ExprFloatSymtabFind
 #define  ExprFloatSymtabFind  G238
 #undef  SymtabFind
 #define  SymtabFind  G239
 #undef  SymtabBuiltinFind
 #define  SymtabBuiltinFind  G240
 #undef  TextCreate
 #define  TextCreate  G241
 #undef  TextAppend
 #define  TextAppend  G242
 #undef TextApply
 #define TextApply  G243
 #undef TileValue
 #define TileValue  G244
 #undef model2text
 #define model2text  G245
 #undef prim2text
 #define prim2text  G246
 #undef world2text
 #define world2text  G247
 #undef  TorusCreate
 #define  TorusCreate  G248
 #undef TorusIntersect
 #define TorusIntersect  G249
 #undef TorusEnter
 #define TorusEnter  G250
 #undef TorusNormal
 #define TorusNormal  G251
 #undef TorusBounds
 #define TorusBounds  G252
 #undef TorusStats
 #define TorusStats  G253
 #undef  TorusName
 #define  TorusName  G254
 #undef  TorusMethods
 #define  TorusMethods  G255
 #undef MatrixMult
 #define MatrixMult  G256
 #undef MatrixCopy
 #define MatrixCopy  G257
 #undef MatrixInit
 #define MatrixInit  G258
 #undef MatrixInvert
 #define MatrixInvert  G259
 #undef TransCopy
 #define TransCopy  G260
 #undef TransInit
 #define TransInit  G261
 #undef TransInvert
 #define TransInvert  G262
 #undef TransCompose
 #define TransCompose  G263
 #undef VecTransform
 #define VecTransform  G264
 #undef PointTransform
 #define PointTransform  G265
 #undef NormalTransform
 #define NormalTransform  G266
 #undef  TransCreate
 #define  TransCreate  G267
 #undef  MatrixCreate
 #define  MatrixCreate  G268
 #undef RayTransform
 #define RayTransform  G269
 #undef  TranslateCreate
 #define  TranslateCreate  G270
 #undef  TranslateMethods
 #define  TranslateMethods  G271
 #undef TranslatePropagate
 #define TranslatePropagate  G272
 #undef  TriangleCreate
 #define  TriangleCreate  G273
 #undef TriangleIntersect
 #define TriangleIntersect  G274
 #undef TriangleNormal
 #define TriangleNormal  G275
 #undef TriangleBounds
 #define TriangleBounds  G276
 #undef TriangleUV
 #define TriangleUV  G277
 #undef TriangleStats
 #define TriangleStats  G278
 #undef  TriangleMethods
 #define  TriangleMethods  G279
 #undef VecCoordSys
 #define VecCoordSys  G280
 #undef VecNormCross
 #define VecNormCross  G281
 #undef VecNormalize
 #define VecNormalize  G282
 #undef  WindyCreate
 #define  WindyCreate  G283
 #undef WindyApply
 #define WindyApply  G284
 #undef  WoodCreate
 #define  WoodCreate  G285
 #undef WoodApply
 #define WoodApply  G286
 #undef FullySampleScanline
 #define FullySampleScanline G287
 #undef SamplingSetTime
 #define SamplingSetTime     G288
 #undef TriangleMethodRegister
 #define TriangleMethodRegister G289
 #undef SampleScreenFiltered
 #define SampleScreenFiltered G290
 #undef RSOptionsList
 #define  RSOptionsList G291
 #undef  SolveQuartic
 #define SolveQuartic G292
 #undef  Metapoints
 #define Metapoints G293
