/*
 * version.c
 * $Log:	version.c,v $
 * Revision 4.0  91/07/17  14:52:08  kolb
 * Initial version.
 */
#include "rayshade.h"
#include "patchlev.h"
#include "stats.h"
 
void
VersionPrint()
{
	extern char rcsid[];
 
	fprintf(Stats.fstats,
		"rayshade: %s\nPatch level %d\n", rcsid, PATCHLEVEL);
}
